/* m_svsnick.c
 * Allows a server to change the nick of another user.
 * Copyright (C) 2003 by MusIRCd Development.
 *
 * $Id: m_svsnick.c,v 1.4 2003/10/19 00:30:09 musirc Exp $
 */

#include "handlers.h"
#include "client.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "hash.h"
#include "s_serv.h"
#include "s_conf.h"
#include "send.h"
#include "resv.h"
#include "modules.h"
#include "common.h"
#include "packet.h"

static void ms_svsnick(struct Client *, struct Client *, int, char**);
static int clean_nick_name(char *);
static void set_svsnick_capab(void);
static void unset_svsnick_capab(void);

#define CAP_SVSNICK 0x00400000

struct Message svsnick_msgtab = {
  "SVSNICK", 0, 0, 3, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, ms_svsnick, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&svsnick_msgtab);
  set_svsnick_capab();
  add_capability("SVSNICK", CAP_SVSNICK, 1);
}

void
_moddeinit(void)
{
  mod_del_cmd(&svsnick_msgtab);
  unset_svsnick_capab();
  delete_capability("SVSNICK");
}

const char *_version = "$Revision: 1.4 $";
#endif

/* ms_svsnick()
 *
 *     parv[0] = sender prefix
 *     parv[1] = oldnick
 *     parv[2] = newnick
 */
static void
ms_svsnick(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
{
  char oldnick[NICKLEN];
  char newnick[NICKLEN];
  struct Client *oldnickname;
  struct Client *newnickname;
  
  if (!IsServer(source_p))
    return;

  if(parc < 3 || EmptyString(parv[1]) || EmptyString(parv[2]))
  {
    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
               me.name, source_p->name);
    return;
  }

  /* terminate nick to NICKLEN */
  strlcpy(oldnick, parv[1], sizeof(oldnick));
  strlcpy(newnick, parv[2], sizeof(newnick));

  if (!clean_nick_name(oldnick) || !clean_nick_name(newnick))
  {
    sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME),
               me.name, source_p->name, oldnick);
    return;
  }

  if (find_matching_name_conf(NRESV_TYPE, newnick, NULL, NULL, 0) &&
     !(IsOper(source_p) && ConfigFileEntry.oper_pass_resv))
  {
    sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE),
               me.name, source_p->name, newnick);
    return;
  }

  if (!(oldnickname = (find_client(oldnick))))
  {
    sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, oldnick);
    return;
  }

  if ((newnickname = (find_client(newnick))))
  {
    sendto_one(source_p, form_str(ERR_NICKNAMEINUSE), me.name,
               source_p->name, newnick);    
    return;
  }

  if (MyConnect(oldnickname))
  {
    if (!IsFloodDone(oldnickname))
      flood_endgrace(oldnickname);

    if ((newnickname = find_client(oldnick)))
    {
      if(newnickname == oldnickname)
      {
        /* check the nick isnt exactly the same */
        if(strcmp(oldnickname->name, newnick))
        {
          change_local_nick(oldnickname->servptr, oldnickname, newnick);
          return;
        }
        else 
        {
          /* client is doing :old NICK old
	   * ignore it..
	   */
          return;
        }
      }
      /* if the client that has the nick isnt registered yet (nick but no
       * user) then drop the unregged client
       */
      if(IsUnknown(newnickname))
      {
        exit_client(NULL, newnickname, &me, "Nick Collision");
        change_local_nick(oldnickname->servptr, oldnickname, newnick);
        return;
      }
      else
      {
        sendto_one(source_p, form_str(ERR_NICKNAMEINUSE), me.name,
                   source_p->name, newnick);
        return;
      }
    }
    else
    {
      if(!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL))
        sendto_one(uplink, ":%s NBURST %s %s %s", me.name, newnick,
                   newnick, oldnickname->name);
      else
        change_local_nick(oldnickname->servptr, oldnickname, newnick);
      return;
    }
  }
  else
  {
    sendto_server(client_p, source_p, NULL, CAP_TS6|CAP_SVSNICK, NOCAPS, NOFLAGS,
                  ":%s SVSNICK %s :%s", me.id, oldnick, newnick);
    sendto_server(client_p, source_p, NULL, CAP_SVSNICK, CAP_TS6, NOFLAGS,
                  ":%s SVSNICK %s :%s", me.name, oldnick, newnick);
  }
}

static int
clean_nick_name(char *nick)
{
  if(nick == NULL)
    return 0;

  /* nicks cant start with a digit or - or be 0 length */
  if (*nick == '-' || IsDigit(*nick) || *nick == '\0')
    return 0;

  for(; *nick; nick++)
    if(!IsNickChar(*nick))
      return 0;

  return 1;
}

static void
set_svsnick_capab(void)
{
  default_server_capabs |= CAP_SVSNICK;
}

static void
unset_svsnick_capab(void)
{
  default_server_capabs &= ~CAP_SVSNICK;
}
